@@ -129,6 +129,7 @@ static bool file_exists (char *filename) {
 blosc2_schunk* blosc2_schunk_new(blosc2_storage *storage) {
   blosc2_schunk* schunk = calloc(1, sizeof(blosc2_schunk));
   schunk->version = 0;     /* pre-first version */
+  schunk->view = false;  /* not a view by default */
 
   // Get the storage with proper defaults
   schunk->storage = get_new_storage(storage, &BLOSC2_CPARAMS_DEFAULTS, &BLOSC2_DPARAMS_DEFAULTS, &BLOSC2_IO_DEFAULTS);
@@ -253,6 +254,8 @@ blosc2_schunk* blosc2_schunk_copy(blosc2_schunk *schunk, blosc2_storage *storage
     BLOSC_TRACE_ERROR("Can not create a new schunk");
     return NULL;
   }
+  // Set the chunksize for the schunk, as it cannot be derived from storage
+  new_schunk->chunksize = schunk->chunksize;
 
   // Copy metalayers
   for (int nmeta = 0; nmeta < schunk->nmetalayers; ++nmeta) {
@@ -313,16 +316,34 @@ blosc2_schunk* blosc2_schunk_copy(blosc2_schunk *schunk, blosc2_storage *storage
 
 /* Open an existing super-chunk that is on-disk (no copy is made). */
 blosc2_schunk* blosc2_schunk_open_udio(const char* urlpath, const blosc2_io *udio) {
+  return blosc2_schunk_open_offset_udio(urlpath, 0, udio);
+}
+
+blosc2_schunk* blosc2_schunk_open_offset_udio(const char* urlpath, int64_t offset, const blosc2_io *udio) {
   if (urlpath == NULL) {
     BLOSC_TRACE_ERROR("You need to supply a urlpath.");
     return NULL;
   }
 
-  blosc2_frame_s* frame = frame_from_file_offset(urlpath, udio, 0);
+  blosc2_frame_s* frame = frame_from_file_offset(urlpath, udio, offset);
   if (frame == NULL) {
+    blosc2_io_cb *io_cb = blosc2_get_io_cb(udio->id);
+    if (io_cb == NULL) {
+        BLOSC_TRACE_ERROR("Error getting the input/output API");
+        return NULL;
+    }
+    int rc = io_cb->destroy(udio->params);
+    if (rc < 0) {
+      BLOSC_TRACE_ERROR("Cannot destroy the input/output object.");
+    }
     return NULL;
   }
   blosc2_schunk* schunk = frame_to_schunk(frame, false, udio);
+  if (schunk == NULL) {
+    frame_free(frame);
+    BLOSC_TRACE_ERROR("Error converting frame to super-chunk");
+    return NULL;
+  }
 
   // Set the storage with proper defaults
   size_t pathlen = strlen(urlpath);
@@ -338,24 +359,7 @@ blosc2_schunk* blosc2_schunk_open(const char* urlpath) {
 }
 
 blosc2_schunk* blosc2_schunk_open_offset(const char* urlpath, int64_t offset) {
-  if (urlpath == NULL) {
-    BLOSC_TRACE_ERROR("You need to supply a urlpath.");
-    return NULL;
-  }
-
-  blosc2_frame_s* frame = frame_from_file_offset(urlpath, &BLOSC2_IO_DEFAULTS, offset);
-  if (frame == NULL) {
-    return NULL;
-  }
-  blosc2_schunk* schunk = frame_to_schunk(frame, false, &BLOSC2_IO_DEFAULTS);
-
-  // Set the storage with proper defaults
-  size_t pathlen = strlen(urlpath);
-  schunk->storage->urlpath = malloc(pathlen + 1);
-  strcpy(schunk->storage->urlpath, urlpath);
-  schunk->storage->contiguous = !frame->sframe;
-
-  return schunk;
+  return blosc2_schunk_open_offset_udio(urlpath, offset, &BLOSC2_IO_DEFAULTS);
 }
 
 int64_t blosc2_schunk_to_buffer(blosc2_schunk* schunk, uint8_t** dest, bool* needs_free) {
@@ -401,7 +405,8 @@ int64_t frame_to_file(blosc2_frame_s* frame, const char* urlpath) {
     return BLOSC2_ERROR_PLUGIN_IO;
   }
   void* fp = io_cb->open(urlpath, "wb", frame->schunk->storage->io);
-  int64_t nitems = io_cb->write(frame->cframe, frame->len, 1, fp);
+  int64_t io_pos = 0;
+  int64_t nitems = io_cb->write(frame->cframe, frame->len, 1, io_pos, fp);
   io_cb->close(fp);
   return nitems * frame->len;
 }
@@ -415,18 +420,11 @@ int64_t append_frame_to_file(blosc2_frame_s* frame, const char* urlpath) {
         return BLOSC2_ERROR_PLUGIN_IO;
     }
     void* fp = io_cb->open(urlpath, "ab", frame->schunk->storage->io);
-    int64_t offset;
 
-# if (UNIX)
-    offset = io_cb->tell(fp);
-# else
-    io_cb->seek(fp, 0, SEEK_END);
-    offset = io_cb->tell(fp);
-# endif
-
-    io_cb->write(frame->cframe, frame->len, 1, fp);
+    int64_t io_pos = io_cb->size(fp);
+    io_cb->write(frame->cframe, frame->len, 1, io_pos, fp);
     io_cb->close(fp);
-    return offset;
+    return io_pos;
 }
 
 
@@ -494,7 +492,10 @@ int64_t blosc2_schunk_append_file(blosc2_schunk* schunk, const char* urlpath) {
 
 /* Free all memory from a super-chunk. */
 int blosc2_schunk_free(blosc2_schunk *schunk) {
-  if (schunk->data != NULL) {
+  int err = 0;
+
+  // If it is a view, the data belongs to original array and should not be freed
+  if (schunk->data != NULL && !schunk->view) {
     for (int i = 0; i < schunk->nchunks; i++) {
       free(schunk->data[i]);
     }
@@ -521,6 +522,15 @@ int blosc2_schunk_free(blosc2_schunk *schunk) {
   }
 
   if (schunk->storage != NULL) {
+    blosc2_io_cb *io_cb = blosc2_get_io_cb(schunk->storage->io->id);
+    if (io_cb != NULL) {
+      int rc = io_cb->destroy(schunk->storage->io->params);
+      if (rc < 0) {
+        BLOSC_TRACE_ERROR("Could not free the I/O resources.");
+        err = 1;
+      }
+    }
+
     if (schunk->storage->urlpath != NULL) {
       free(schunk->storage->urlpath);
     }
@@ -530,7 +540,8 @@ int blosc2_schunk_free(blosc2_schunk *schunk) {
     free(schunk->storage);
   }
 
-  if (schunk->frame != NULL) {
+  // If it is a view, the frame belongs to original array and should not be freed
+  if (schunk->frame != NULL && !schunk->view) {
     frame_free((blosc2_frame_s *) schunk->frame);
   }
 
@@ -548,7 +559,7 @@ int blosc2_schunk_free(blosc2_schunk *schunk) {
 
   free(schunk);
 
-  return 0;
+  return err;
 }
 
 
@@ -1120,9 +1131,9 @@ int blosc2_schunk_decompress_chunk(blosc2_schunk *schunk, int64_t nchunk,
 */
 int blosc2_schunk_get_chunk(blosc2_schunk *schunk, int64_t nchunk, uint8_t **chunk, bool *needs_free) {
   if (schunk->dctx->threads_started > 1) {
-    pthread_mutex_lock(&schunk->dctx->nchunk_mutex);
+    blosc2_pthread_mutex_lock(&schunk->dctx->nchunk_mutex);
     schunk->current_nchunk = nchunk;
-    pthread_mutex_unlock(&schunk->dctx->nchunk_mutex);
+    blosc2_pthread_mutex_unlock(&schunk->dctx->nchunk_mutex);
   }
   else {
     schunk->current_nchunk = nchunk;
@@ -1166,9 +1177,9 @@ int blosc2_schunk_get_chunk(blosc2_schunk *schunk, int64_t nchunk, uint8_t **chu
 */
 int blosc2_schunk_get_lazychunk(blosc2_schunk *schunk, int64_t nchunk, uint8_t **chunk, bool *needs_free) {
   if (schunk->dctx->threads_started > 1) {
-    pthread_mutex_lock(&schunk->dctx->nchunk_mutex);
+    blosc2_pthread_mutex_lock(&schunk->dctx->nchunk_mutex);
     schunk->current_nchunk = nchunk;
-    pthread_mutex_unlock(&schunk->dctx->nchunk_mutex);
+    blosc2_pthread_mutex_unlock(&schunk->dctx->nchunk_mutex);
   }
   else {
     schunk->current_nchunk = nchunk;
