@@ -8,7 +8,8 @@
   See LICENSE.txt for details about copyright and rights to use.
 **********************************************************************/
 
-#include "shuffle.h" /* needs to be included first to define macros */
+#include "blosc2.h" /* needs to be included first to define macros */
+#include "shuffle.h"
 
 /*  Include hardware-accelerated shuffle/unshuffle routines based on
     the target architecture. Note that a target architecture may support
@@ -45,22 +46,21 @@
 
 #include "shuffle-generic.h"
 #include "bitshuffle-generic.h"
-#include "blosc2.h"
 
 #include <stdio.h>
 #include <string.h>
 
 // __builtin_cpu_supports() fixed in GCC 8: https://gcc.gnu.org/bugzilla/show_bug.cgi?id=85100
 // Also, clang added support for it in clang 10 at very least (and possibly since 3.8)
-#if (defined(__clang__) && (__clang_major__ >= 10)) || \
+#if (defined(__clang__) && (__clang_major__ >= 10)) && !(defined(__APPLE__) && defined(__x86_64__) && defined(BUILD_STATIC)) || \
     (defined(__GNUC__) && defined(__GNUC_MINOR__) && __GNUC__ >= 8)
 #define HAVE_CPU_FEAT_INTRIN
 #endif
 
 
 /*  Define function pointer types for shuffle/unshuffle routines. */
-typedef void(* shuffle_func)(const int32_t, const int32_t, const uint8_t*, const uint8_t*);
-typedef void(* unshuffle_func)(const int32_t, const int32_t, const uint8_t*, const uint8_t*);
+typedef void(* shuffle_func)(const int32_t, const int32_t, const uint8_t*, uint8_t*);
+typedef void(* unshuffle_func)(const int32_t, const int32_t, const uint8_t*, uint8_t*);
 // For bitshuffle, everything is done in terms of size_t and int64_t (return value)
 // and although this is not strictly necessary for Blosc, it does not hurt either
 typedef int64_t(* bitshuffle_func)(const void*, void*, const size_t, const size_t);
@@ -268,7 +268,7 @@ static blosc_cpu_features blosc_get_cpu_features(void) {
 #if defined(__aarch64__)
   /* aarch64 always has NEON */
   cpu_features |= BLOSC_HAVE_NEON;
-#else
+#elif defined(__linux__)
   if (getauxval(AT_HWCAP) & HWCAP_ARM_NEON) {
     cpu_features |= BLOSC_HAVE_NEON;
   }
@@ -417,42 +417,58 @@ void init_shuffle_implementation(void) {
 
 /* Shuffle a block by dynamically dispatching to the appropriate
    hardware-accelerated routine at run-time. */
-void
-shuffle(const int32_t bytesoftype, const int32_t blocksize,
-        const uint8_t* _src, const uint8_t* _dest) {
+int32_t
+blosc2_shuffle(const int32_t typesize, const int32_t blocksize,
+               const void* src, void* dest) {
   /* Initialize the shuffle implementation if necessary. */
   init_shuffle_implementation();
 
+  if (typesize < 1 || typesize > 256 || blocksize < 0) {
+    return BLOSC2_ERROR_INVALID_PARAM;
+  }
+
   /* The implementation is initialized.
      Dispatch to its shuffle routine. */
-  (host_implementation.shuffle)(bytesoftype, blocksize, _src, _dest);
+  (host_implementation.shuffle)(typesize, blocksize, src, dest);
+
+  return blocksize;
 }
 
 /* Unshuffle a block by dynamically dispatching to the appropriate
    hardware-accelerated routine at run-time. */
-void
-unshuffle(const int32_t bytesoftype, const int32_t blocksize,
-          const uint8_t* _src, const uint8_t* _dest) {
+int32_t
+blosc2_unshuffle(const int32_t typesize, const int32_t blocksize,
+                 const void* src, void* dest) {
   /* Initialize the shuffle implementation if necessary. */
   init_shuffle_implementation();
 
+  if (typesize < 1 || typesize > 256 || blocksize < 0) {
+    return BLOSC2_ERROR_INVALID_PARAM;
+  }
+
   /* The implementation is initialized.
      Dispatch to it's unshuffle routine. */
-  (host_implementation.unshuffle)(bytesoftype, blocksize, _src, _dest);
+  (host_implementation.unshuffle)(typesize, blocksize, src, dest);
+
+  return blocksize;
 }
 
 /*  Bit-shuffle a block by dynamically dispatching to the appropriate
     hardware-accelerated routine at run-time. */
 int32_t
-bitshuffle(const int32_t bytesoftype, const int32_t blocksize,
-           const uint8_t *_src, const uint8_t *_dest) {
+blosc2_bitshuffle(const int32_t typesize, const int32_t blocksize,
+                  const void* src, void* dest) {
   /* Initialize the shuffle implementation if necessary. */
   init_shuffle_implementation();
-  size_t size = blocksize / bytesoftype;
+  size_t size = blocksize / typesize;
+
+  if (typesize < 1 || typesize > 256 || blocksize < 0) {
+    return BLOSC2_ERROR_INVALID_PARAM;
+  }
+
   /* bitshuffle only supports a number of elements that is a multiple of 8. */
   size -= size % 8;
-  int ret = (int) (host_implementation.bitshuffle)
-      ((const void *) _src, (void *) _dest, size, bytesoftype);
+  int ret = (int) (host_implementation.bitshuffle)(src, dest, size, typesize);
   if (ret < 0) {
     // Some error in bitshuffle (should not happen)
     BLOSC_TRACE_ERROR("the impossible happened: the bitshuffle filter failed!");
@@ -460,20 +476,20 @@ bitshuffle(const int32_t bytesoftype, const int32_t blocksize,
   }
 
   // Copy the leftovers
-  size_t offset = size * bytesoftype;
-  memcpy((void *) (_dest + offset), (void *) (_src + offset), blocksize - offset);
+  size_t offset = size * typesize;
+  memcpy((uint8_t *) dest + offset, (const uint8_t *) src + offset, blocksize - offset);
 
   return blocksize;
 }
 
 /*  Bit-unshuffle a block by dynamically dispatching to the appropriate
     hardware-accelerated routine at run-time. */
-int32_t bitunshuffle(const int32_t bytesoftype, const int32_t blocksize,
-                     const uint8_t *_src, const uint8_t *_dest,
+int32_t bitunshuffle(const int32_t typesize, const int32_t blocksize,
+                     const void* src, void* dest,
                      const uint8_t format_version) {
   /* Initialize the shuffle implementation if necessary. */
   init_shuffle_implementation();
-  size_t size = blocksize / bytesoftype;
+  size_t size = blocksize / typesize;
 
   if (format_version == 2) {
     /* Starting from version 3, bitshuffle() works differently */
@@ -481,34 +497,37 @@ int32_t bitunshuffle(const int32_t bytesoftype, const int32_t blocksize,
       /* The number of elems is a multiple of 8 which is supported by
          bitshuffle. */
       int ret = (int) (host_implementation.bitunshuffle)
-          ((const void *) _src, (void *) _dest, blocksize / bytesoftype, bytesoftype);
+          ((const void *) src, (void *) dest, blocksize / typesize, typesize);
       if (ret < 0) {
         // Some error in bitshuffle (should not happen)
         BLOSC_TRACE_ERROR("the impossible happened: the bitunshuffle filter failed!");
         return ret;
       }
-      /* Copy the leftovers (we do so starting from c-blosc 1.18 on) */
-      size_t offset = size * bytesoftype;
-      memcpy((void *) (_dest + offset), (void *) (_src + offset), blocksize - offset);
     }
     else {
-      memcpy((void *) _dest, (void *) _src, blocksize);
+      memcpy(dest, src, blocksize);
     }
   }
   else {
     /* bitshuffle only supports a number of bytes that is a multiple of 8. */
     size -= size % 8;
-    int ret = (int) (host_implementation.bitunshuffle)
-        ((const void *) _src, (void *) _dest, size, bytesoftype);
+    int ret = (int) (host_implementation.bitunshuffle)(src, dest, size, typesize);
     if (ret < 0) {
       BLOSC_TRACE_ERROR("the impossible happened: the bitunshuffle filter failed!");
       return ret;
     }
 
     /* Copy the leftovers */
-    size_t offset = size * bytesoftype;
-    memcpy((void *) (_dest + offset), (void *) (_src + offset), blocksize - offset);
+    size_t offset = size * typesize;
+    memcpy((uint8_t *) dest + offset, (const uint8_t *) src + offset, blocksize - offset);
   }
 
   return blocksize;
 }
+
+/* Stub public API that redirects to internal implementation. */
+int32_t
+blosc2_bitunshuffle(const int32_t typesize, const int32_t blocksize,
+                    const void * src, void* dest) {
+  return bitunshuffle(typesize, blocksize, src, dest, BLOSC2_VERSION_FORMAT);
+}
